package gov.nih.nlm.nls.lexCheck.CkLib;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
/*****************************************************************************
* This class provides a Java object to carry check information.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2016
****************************************************************************/
public class CheckObject
{
    // public constructor
    /**
    * Create a default CheckObject
    *
    * @param   startStr  the start string of a line
    * @param   startErrMsg   the error msg for the start string problem
    * @param   fillerErrMsg  the error msg for filler 
    * @param   nextState     the next checking state
    * @param   nextStartStrs  the possible start string for the next line
    * @param   fillerFormat   the legal format of filler
    */
    public CheckObject(String startStr, int startErrMsg, int fillerErrMsg,
        int nextState, HashSet<String> nextStartStrs, CheckFormat fillerFormat)
    {
        startStr_ = startStr;
        delim_ = "=";
        startErrMsg_ = startErrMsg;
        fillerErrMsg_ = fillerErrMsg;
        nextState_ = nextState;
        nextStartStrs_ = nextStartStrs;
        nextLine_ = null;
        fillerFormat_ = fillerFormat;
        if(startStr != null)
        {
            isTab_ = startStr.startsWith("\t");
        }
    }
    /**
    * Create a default CheckObject
    *
    * @param   startStr  the start string of a line
    * @param   startErrMsg   the error msg for the start string problem
    * @param   fillerErrMsg  the error msg for filler 
    * @param   nextState     the next checking state
    * @param   nextStartStrs  the possible start string for the next line
    * @param   nextLine       the possible next line
    * @param   fillerFormat   the legal format of filler
    */
    public CheckObject(String startStr, int startErrMsg, int fillerErrMsg,
        int nextState, HashSet<String> nextStartStrs, HashSet<String> nextLine,
        CheckFormat fillerFormat)
    {
        this(startStr, startErrMsg, fillerErrMsg, nextState, nextStartStrs,
            fillerFormat);
        nextLine_ = nextLine;
    }
    /**
    * Create a default CheckObject
    *
    * @param   startStr  the start string of a line
    * @param   startErrMsg   the error msg for the start string problem
    * @param   fillerErrMsg  the error msg for filler 
    * @param   nextState     the next checking state
    * @param   nextStartStrs  the possible start string for the next line
    * @param   fillerFormat   the legal format of filler
    * @param   delim          the deliminator for tokenization
    */
    public CheckObject(String startStr, int startErrMsg, int fillerErrMsg,
        int nextState, HashSet<String> nextStartStrs, CheckFormat fillerFormat, 
        String delim)
    {
        this(startStr, startErrMsg, fillerErrMsg, nextState, nextStartStrs,
            fillerFormat);
        delim_ = delim;
    }
    /**
    * Create a default CheckObject
    *
    * @param   startStr  the start string of a line
    * @param   startErrMsg   the error msg for the start string problem
    * @param   fillerErrMsg  the error msg for filler 
    * @param   nextState     the next checking state
    * @param   nextStartStrs  the possible start string for the next line
    * @param   nextLine       the possible next line
    * @param   fillerFormat   the legal format of filler
    * @param   delim          the deliminator for tokenization
    */
    public CheckObject(String startStr, int startErrMsg, int fillerErrMsg,
        int nextState, HashSet<String> nextStartStrs, HashSet<String> nextLine,
        CheckFormat fillerFormat, String delim)
    {
        this(startStr, startErrMsg, fillerErrMsg, nextState, nextStartStrs,
            nextLine, fillerFormat);
        delim_ = delim;
    }
    // public methods
    /**
    * Get the legal start string from this CheckObject
    *
    * @return   legal start string from this CheckObject
    */
    public String GetStartStr()
    {
        return startStr_;
    }
    /**
    * Get the deliminator string from this CheckObject
    *
    * @return   the deliminator from this CheckObject
    */
    public String GetDelim()
    {
        return delim_;
    }
    /**
    * Get the error massage of the starting string problem
    *
    * @return   the index of error message for starting string problem
    */
    public int GetStartErrMsg()
    {
        return startErrMsg_;
    }
    /**
    * Get the error massage of the filler problem
    *
    * @return   the index of error message for filler problem
    */
    public int GetFillerErrMsg()
    {
        return fillerErrMsg_;
    }
    /**
    * Get the next checking state
    *
    * @return   the next checking state
    */
    public int GetNextState()
    {
        return nextState_;
    }
    /**
    * Get the collection of possible starting strings of next line
    *
    * @return   a hashset collection of possible starting strings of next line
    */
    public HashSet<String> GetNextStartStrs()
    {
        return nextStartStrs_;
    }
    /**
    * Get the collection of possible strings of next line
    *
    * @return   a hashset collection of possible strings of next line
    */
    public HashSet<String> GetNextLine()
    {
        return nextLine_;
    }
    /**
    * Check if the starting string of next line is legal
    *
	* @param  line  input line of current checking lexical record
	*
    * @return  true or false if the start string of next line is legal or not
    */
    public boolean IsNextStartStr(String line)
    {
        boolean flag = false;
          for(Iterator<String> it = nextStartStrs_.iterator(); it.hasNext();)
        {
            String itNext = it.next();
            if(line.startsWith(itNext) == true)
            {
                flag = true;
                //check if it belong to nextLine_: trademark= & trademark
                // "trademark=" has the same start as "trademark"
                // However, they can only appear once.
                if(nextLine_ != null)
                {
                    for(Iterator<String> it2 = nextLine_.iterator(); it2.hasNext();)
                    {
                        String it2Next = it2.next();
                        if((line.startsWith(it2Next) == true)
                        && (line.equals(it2Next) == false))
                        {
                            flag = false;
                            break;
                        }
                    }
                }
                break;
            }
        }
       
        return flag;
    }
    /**
    * Get the legal format of a filler
    *
    * @return   the legal format of a filler
    */
    public CheckFormat GetFillerFormat()
    {
        return fillerFormat_;
    }
    /**
    * Check if the line starts with a tab
    *
    * @return   true or false if the line starts with a tab
    */
    public boolean IsTab()
    {
        return isTab_;
    }
    // data member
    private String startStr_ = null;
    private String delim_ = "=";
    private int startErrMsg_ = ErrMsg.NONE;
    private int fillerErrMsg_ = ErrMsg.NONE;
    private int nextState_ = CheckSt.NO_CHANGE;
    private HashSet<String> nextStartStrs_ = null;    // collection of following startStrs
    private HashSet<String> nextLine_ = null;    // used in trademark= and trademark
    private CheckFormat fillerFormat_ = null;
    private boolean isTab_ = false;
}
